from dialog_bot_sdk.entities.media.FileLocation import FileLocation
from dialog_api import media_and_files_pb2


class AvatarImage:
    def __init__(self, file_location: FileLocation, file_size: int, height: int, width: int) -> None:
        self.file_location = file_location
        self.file_size = file_size
        self.height = height
        self.width = width

    @classmethod
    def from_api(cls, avatar_image: media_and_files_pb2.AvatarImage) -> 'AvatarImage':
        return cls(FileLocation(avatar_image.file_location.file_id, avatar_image.file_location.access_hash),
                   avatar_image.file_size, avatar_image.height, avatar_image.width)

    def __dict__(self):
        return {"file_location": self.file_location.__dict__(), "file_size": self.file_size,
                "height": self.height, "width": self.width}

    def __str__(self):
        return "AvatarImage({})".format(self.__dict__())


class Avatar:
    def __init__(self, small_image: AvatarImage, large_image: AvatarImage, full_image: AvatarImage) -> None:
        self.small_image = small_image
        self.large_image = large_image
        self.full_image = full_image

    @classmethod
    def from_api(cls, avatar: media_and_files_pb2.Avatar) -> 'Avatar':
        return cls(AvatarImage.from_api(avatar.small_image),
                   AvatarImage.from_api(avatar.large_image),
                   AvatarImage.from_api(avatar.full_image))

    def __dict__(self):
        return {"small_image": self.small_image.__dict__(), "large_image": self.large_image.__dict__(),
                "full_image": self.full_image.__dict__()}

    def __str__(self):
        return "Avatar({})".format(self.__dict__())



from typing import List

from dialog_api import messaging_pb2

from dialog_bot_sdk.entities.media.AudioMedia import AudioMedia
from dialog_bot_sdk.entities.media.ImageMedia import ImageMedia
from dialog_bot_sdk.entities.media.InteractiveMediaGroup import InteractiveMediaGroup
from dialog_bot_sdk.entities.media.WebpageMedia import WebPageMedia


class MessageMedia:
    def __init__(self, web_page: WebPageMedia = None, image: ImageMedia = None, audio: AudioMedia = None,
                 actions: List[InteractiveMediaGroup] = None) -> None:
        self.web_page = web_page
        self.image = image
        self.audio = audio
        self.actions = actions

    def to_api(self) -> messaging_pb2.MessageMedia:
        web, image, audio, actions = None, None, None, []
        if self.web_page is not None:
            web = self.web_page.to_api()
        if self.image is not None:
            image = self.image.to_api()
        if self.audio is not None:
            audio = self.audio.to_api()
        if self.actions is not None:
            actions = [x.to_api() for x in self.actions]
        return messaging_pb2.MessageMedia(webpage=web, image=image, audio=audio, actions=actions)

    @classmethod
    def from_api(cls, message_media: messaging_pb2.MessageMedia) -> 'MessageMedia':
        return cls(WebPageMedia.from_api(message_media.webpage), ImageMedia.from_api(message_media.image),
                   AudioMedia.from_api(message_media.audio),
                   [InteractiveMediaGroup.from_api(x) for x in message_media.actions])

    def __dict__(self):
        return {"web_page": self.web_page.__dict__(), "image": self.image.__dict__(),
                "audio": self.audio.__dict__(), "actions": [x.__dict__() for x in self.actions]}

    def __str__(self):
        return "MessageMedia({})".format(self.__dict__())


class TextMessage:
    def __init__(self, text: str, media: List[MessageMedia],
                 mentions: List[int]) -> None:
        self.text = text
        self.media = media
        self.mentions = mentions

    @classmethod
    def from_api(cls, text_message: messaging_pb2.TextMessage) -> 'TextMessage':
        return cls(text_message.text,
                   [MessageMedia.from_api(x) for x in text_message.media], [x for x in text_message.mentions])

    def __dict__(self):
        return {"text": self.text, "media": [x.__dict__() for x in self.media], "mentions": self.mentions}

    def __str__(self):
        return "TextMessage({})".format(self.__dict__())



from typing import List
from dialog_api import groups_pb2
from dialog_api.groups_pb2 import GROUPADMINPERMISSION_EDITSHORTNAME, GROUPADMINPERMISSION_INVITE, \
    GROUPADMINPERMISSION_KICK, GROUPADMINPERMISSION_UPDATEINFO, GROUPADMINPERMISSION_SETPERMISSIONS, \
    GROUPADMINPERMISSION_EDITMESSAGE, GROUPADMINPERMISSION_DELETEMESSAGE, GROUPADMINPERMISSION_GETINTEGRATIONTOKEN, \
    GROUPADMINPERMISSION_SENDMESSAGE, GROUPADMINPERMISSION_PINMESSAGE, GROUPADMINPERMISSION_VIEWMEMBERS, \
    GROUPADMINPERMISSION_LEAVE, GROUPADMINPERMISSION_TARGETING


PERMISSION_TO_STRING = {
    GROUPADMINPERMISSION_EDITSHORTNAME: "edit short_name",
    GROUPADMINPERMISSION_INVITE: "invite",
    GROUPADMINPERMISSION_KICK: "kick",
    GROUPADMINPERMISSION_UPDATEINFO: "update info",
    GROUPADMINPERMISSION_SETPERMISSIONS: "set permissions",
    GROUPADMINPERMISSION_EDITMESSAGE: "edit message",
    GROUPADMINPERMISSION_DELETEMESSAGE: "delete message",
    GROUPADMINPERMISSION_GETINTEGRATIONTOKEN: "get integration token",
    GROUPADMINPERMISSION_SENDMESSAGE: "send message",
    GROUPADMINPERMISSION_PINMESSAGE: "pen message",
    GROUPADMINPERMISSION_VIEWMEMBERS: "view members",
    GROUPADMINPERMISSION_LEAVE: "leave",
    GROUPADMINPERMISSION_TARGETING: "targeting",
}


class GroupPermission:
    GROUPADMINPERMISSION_EDITSHORTNAME = GROUPADMINPERMISSION_EDITSHORTNAME
    GROUPADMINPERMISSION_INVITE = GROUPADMINPERMISSION_INVITE
    GROUPADMINPERMISSION_KICK = GROUPADMINPERMISSION_KICK
    GROUPADMINPERMISSION_UPDATEINFO = GROUPADMINPERMISSION_UPDATEINFO
    GROUPADMINPERMISSION_SETPERMISSIONS = GROUPADMINPERMISSION_SETPERMISSIONS
    GROUPADMINPERMISSION_EDITMESSAGE = GROUPADMINPERMISSION_EDITMESSAGE
    GROUPADMINPERMISSION_DELETEMESSAGE = GROUPADMINPERMISSION_DELETEMESSAGE
    GROUPADMINPERMISSION_GETINTEGRATIONTOKEN = GROUPADMINPERMISSION_GETINTEGRATIONTOKEN
    GROUPADMINPERMISSION_SENDMESSAGE = GROUPADMINPERMISSION_SENDMESSAGE
    GROUPADMINPERMISSION_PINMESSAGE = GROUPADMINPERMISSION_PINMESSAGE
    GROUPADMINPERMISSION_VIEWMEMBERS = GROUPADMINPERMISSION_VIEWMEMBERS
    GROUPADMINPERMISSION_LEAVE = GROUPADMINPERMISSION_LEAVE
    GROUPADMINPERMISSION_TARGETING = GROUPADMINPERMISSION_TARGETING


class Permissions:
    def __init__(self, user_id: int, permissions: List[GroupPermission]) -> None:
        self.user_id = user_id
        self.permissions = permissions

    @classmethod
    def from_api(cls, permissions: groups_pb2.GroupMemberPermission) -> 'Permissions':
        return cls(permissions.user_id, [x for x in permissions.permissions])

    def __dict__(self):
        return {"user_id": self.user_id, "permissions": [PERMISSION_TO_STRING[x] for x in self.permissions]}

    def __str__(self):
        return "Permissions({})".format(self.__dict__())



import math
import sched
import time
from typing import List, Tuple

from dialog_bot_sdk.entities.ListLoadMode import ListLoadMode
from dialog_bot_sdk.entities.ReferencedEntities import ReferencedEntities
from dialog_bot_sdk.entities.UUID import UUID
from dialog_bot_sdk.entities.UpdateInteractiveMediaEvent import UpdateInteractiveMediaEvent
from dialog_bot_sdk.entities.UpdateMessage import UpdateMessage
from dialog_bot_sdk.entities.message.TextMessage import MessageMedia
from dialog_bot_sdk.interactive_media import InteractiveMediaGroup
from google.protobuf import empty_pb2
import threading
import random
import grpc
import logging

from dialog_bot_sdk.entities.message.Message import Message
from dialog_bot_sdk.entities.Peer import Peer
from dialog_bot_sdk.utils import POOL
from .service import ManagedService
from dialog_api import messaging_pb2, sequence_and_updates_pb2, peers_pb2
from .content import content
import google.protobuf.wrappers_pb2 as wrappers_pb2
from dialog_bot_sdk.utils import get_peer, async_dec, AsyncTask, is_image, get_uuids

SCHEDULER = sched.scheduler(time.time, time.sleep)
MAX_SLEEP_TIME = 30
EXCEPTION_CODES = [1, 13]


class Messaging(ManagedService):
    retry = 0
    timer = 0
    """Main messaging class.
    """
    @async_dec()
    def send_message(self, peer: Peer or AsyncTask, text: str,
                     interactive_media_groups: List[InteractiveMediaGroup] = None,
                     uid: int = None) -> UUID:
        """Send text message to peer.
        Message can contain interactive media groups (buttons, selects etc.).

        :param peer: Peer or AsyncTask (in which located User or Group)
        :param text: message text (not null)
        :param interactive_media_groups: groups of interactive media components (buttons etc.)
        :param uid: send message only for user by id
        :return: UUID (message id)
        """
        peer = get_peer(peer)
        if text == '' or text is None:
            raise AttributeError('Text message must contain some text.')
        out_peer = self.manager.get_out_peer(peer)
        msg = messaging_pb2.MessageContent()
        msg.textMessage.text = text
        if interactive_media_groups is not None:
            for g in interactive_media_groups:
                media = msg.textMessage.media.add()
                g.render(media)
        request = messaging_pb2.RequestSendMessage(
            peer=out_peer,
            deduplication_id=random.randint(0, 100000000),
            message=msg,
            is_only_for_user=uid
        )
        return self.__send_message(request)

    @async_dec()
    def update_message(self, message: Message or AsyncTask, text: str,
                       interactive_media_groups: List[InteractiveMediaGroup] = None) -> None:
        """Update text message or interactive media (buttons, selects etc.).

        :param message: Message or AsyncTask (in which located Message)
        :param text: message text (not null)
        :param interactive_media_groups: groups of interactive media components (buttons etc.)
        :return: None
        """
        try:
            if isinstance(message, AsyncTask):
                message = message.wait()[0]
                if not isinstance(message, Message):
                    raise AttributeError()
        except Exception as e:
            raise AttributeError("if message is AsyncTask class, result must be list of Message")
        msg = messaging_pb2.MessageContent()
        msg.textMessage.text = text
        if interactive_media_groups is not None:
            for g in interactive_media_groups:
                media = msg.textMessage.media.add()
                g.render(media)

        self.__update(message, msg)

    @async_dec()
    def delete(self, message: Message or AsyncTask) -> None:
        """Delete message.

        :param message: Message or AsyncTask (in which located Message)
        :return: None
        """
        msg = messaging_pb2.MessageContent(
            deletedMessage=messaging_pb2.DeletedMessage(is_local=wrappers_pb2.BoolValue(value=False))
        )
        self.__update(message, msg)

    @async_dec()
    def get_messages_by_id(self, mids: List[UUID or AsyncTask]) -> List[Message]:
        """Find and return messages by UUIDs

        :param mids: list of UUID or AsyncTask (in which located UUID)
        :return: list of Messages
        """
        mids = get_uuids(mids)
        request = sequence_and_updates_pb2.RequestGetReferencedEntitites(
                mids=mids
            )
        result = ReferencedEntities.from_api(self.internal.updates.GetReferencedEntitites(request))
        return result.messages

    @async_dec()
    def messages_read(self, peer: Peer or AsyncTask, date: int) -> None:
        """Marking a message and all previous as read

        :param peer: Peer or AsyncTask (in which located User or Group)
        :param date: date of message
        :return: None
        """
        peer = get_peer(peer)
        peer = self.manager.get_out_peer(peer)
        request = messaging_pb2.RequestMessageRead(
            peer=peer,
            date=date
        )
        self.internal.messaging.MessageRead(request)

    @async_dec()
    def send_file(self, peer: Peer or AsyncTask, file: str) -> UUID or None:
        """Send file to peer.

        :param peer: Peer or AsyncTask (in which located User or Group)
        :param file: path to file
        :return: UUID (message id)
        """
        peer = get_peer(peer)
        location = self.internal.uploading.upload_file(file).wait()
        if location is None:
            return None
        location = location.to_api()

        out_peer = self.manager.get_out_peer(peer)
        msg = messaging_pb2.MessageContent()

        msg.documentMessage.CopyFrom(
            content.get_document_content(file, location)
        )

        request = messaging_pb2.RequestSendMessage(
            peer=out_peer,
            deduplication_id=random.randint(0, 100000000),
            message=msg
        )
        return self.__send_message(request)

    @async_dec()
    def send_media(self, peer: Peer or AsyncTask, medias: List[MessageMedia]) -> UUID:
        """Send media to peer.

        :param peer: Peer or AsyncTask (in which located User or Group)
        :param medias: medias (list of MessageMedias)
        :return: UUID (message id)
        """
        peer = get_peer(peer)
        medias = self.__get_medias(medias)
        out_peer = self.manager.get_out_peer(peer)
        text_message = messaging_pb2.TextMessage()
        for media in medias:
            text_message.media.append(media)
        msg = messaging_pb2.MessageContent(textMessage=text_message)
        request = messaging_pb2.RequestSendMessage(
            peer=out_peer,
            deduplication_id=random.randint(0, 100000000),
            message=msg
        )
        return self.__send_message(request)

    @async_dec()
    def send_image(self, peer: Peer or AsyncTask, file: str) -> UUID or None:
        """Send image as image (not as file) to peer.

        :param peer: Peer or AsyncTask (in which located User or Group)
        :param file: path to image file
        :return: UUID (message id)
        """
        peer = get_peer(peer)

        if isinstance(file, str) and not is_image(file):
            raise IOError('File is not an image.')

        location = self.internal.uploading.upload_file(file).wait()
        if location is None:
            return None
        location = location.to_api()
        out_peer = self.manager.get_out_peer(peer)
        msg = messaging_pb2.MessageContent()

        msg.documentMessage.CopyFrom(
            content.get_image_content(file, location)
        )

        request = messaging_pb2.RequestSendMessage(
            peer=out_peer,
            deduplication_id=random.randint(0, 100000000),
            message=msg
        )

        return self.__send_message(request)

    @async_dec()
    def reply(self, peer: Peer or AsyncTask, mids: List[UUID or AsyncTask], text: str = None,
              interactive_media_groups: List[InteractiveMediaGroup] = None) -> UUID:
        """Reply messages to peer. Message can contain interactive media groups (buttons, selects etc.).

        :param peer: Peer or AsyncTask (in which located User or Group)
        :param mids: list of UUIDs
        :param text: message text
        :param interactive_media_groups: groups of interactive media components (buttons etc.)
        :return: UUID (message id)
        """
        peer = get_peer(peer)
        mids = get_uuids(mids)
        if text is None:
            text = ''

        out_peer = self.manager.get_out_peer(peer)
        msg = messaging_pb2.MessageContent()
        msg.textMessage.text = text
        if interactive_media_groups is not None:
            for g in interactive_media_groups:
                media = msg.textMessage.media.add()
                g.render(media)
        request = messaging_pb2.RequestSendMessage(
            peer=out_peer,
            deduplication_id=random.randint(0, 100000000),
            message=msg,
            reply=messaging_pb2.ReferencedMessages(mids=mids)
        )
        return self.__send_message(request)

    @async_dec()
    def forward(self, peer: Peer or AsyncTask, mids: List[UUID or AsyncTask], text: str = None,
                interactive_media_groups: List[InteractiveMediaGroup] = None) -> UUID:
        """Forward messages to peer. Message can contain interactive media groups (buttons, selects etc.).

        :param peer: Peer or AsyncTask (in which located User or Group)
        :param mids: list of UUIDs
        :param text: message text
        :param interactive_media_groups: groups of interactive media components (buttons etc.)
        :return: UUID (message id)
        """
        peer = get_peer(peer)
        mids = get_uuids(mids)
        if text is None:
            text = ''

        out_peer, msg = self.__get_out_peer_and_message(peer, text, interactive_media_groups)
        request = messaging_pb2.RequestSendMessage(
            peer=out_peer,
            deduplication_id=random.randint(0, 100000000),
            message=msg,
            forward=messaging_pb2.ReferencedMessages(mids=mids),
        )
        return self.__send_message(request)

    @async_dec()
    def load_message_history(self, peer: Peer or AsyncTask, date: int = 0,
                             direction: ListLoadMode = ListLoadMode.LISTLOADMODE_BACKWARD,
                             limit: int = 2) -> List[Message]:
        """Load and return messages by peer.

        :param peer: Peer or AsyncTask (in which located User or Group)
        :param date: date of message
        :param direction: ListLoadMode
        :param limit: messages count
        :return: list of Messages
        """
        peer = get_peer(peer)
        out_peer = self.manager.get_out_peer(peer)
        request = messaging_pb2.RequestLoadHistory(
                peer=out_peer,
                date=date,
                load_mode=direction,
                limit=limit
            )
        return [Message.from_api(x) for x in self.internal.messaging.LoadHistory(request).history]

    def on_message_async(self, callback, interactive_media_callback=None) -> None:
        updates_thread = threading.Thread(target=self.on_message, args=(callback, interactive_media_callback))
        updates_thread.start()

    def on_message(self, callback, interactive_media_callback=None, raw_callback=None) -> None:
        """Message receiving event handler.

        :param callback: function that will be called when message received
        :param interactive_media_callback: function that will be called when interactive media action is performed
        :param raw_callback: function to handle any other type of update
        :return: None
        """
        while True:
            try:
                SCHEDULER.enter(min(math.exp(self.retry), MAX_SLEEP_TIME), 1, self.__on_message_schedule,
                                kwargs={'callback': callback,
                                        'interactive_media_callback': interactive_media_callback,
                                        'raw_callback': raw_callback})
                SCHEDULER.run()
            except grpc.RpcError as e:
                logging.error(e)
                if e.details() == 'failed to connect to all addresses':
                    self.timer += min(math.exp(self.retry), MAX_SLEEP_TIME)
                    self.retry += 1
                    continue
                if e.details() in EXCEPTION_CODES:
                    continue

    def __on_message_schedule(self, callback, interactive_media_callback=None, raw_callback=None):
        try:
            self.internal.updates.GetState(sequence_and_updates_pb2.RequestGetState())
            if self.retry:
                logging.info("Server was unavailable {} seconds.".format(int(self.timer)))
                self.timer = 0
                self.retry = 0
        except grpc.RpcError as e:
            raise e
        for update in self.internal.updates.SeqUpdates(empty_pb2.Empty()):
            up = sequence_and_updates_pb2.UpdateSeqUpdate()
            up.ParseFromString(update.update.value)
            if up.WhichOneof('update') == 'updateMessage':
                self.internal.messaging.MessageReceived(messaging_pb2.RequestMessageReceived(
                    peer=self.manager.get_out_peer(up.updateMessage.peer),
                    date=up.updateMessage.date
                ))
                self.internal.messaging.MessageRead(messaging_pb2.RequestMessageRead(
                    peer=self.manager.get_out_peer(up.updateMessage.peer),
                    date=up.updateMessage.date
                ))
                POOL.submit(
                    callback(UpdateMessage.from_api(up.updateMessage))
                )
            elif up.WhichOneof('update') == 'updateInteractiveMediaEvent' and \
                    callable(interactive_media_callback):
                POOL.submit(
                    interactive_media_callback(UpdateInteractiveMediaEvent.from_api(up.updateInteractiveMediaEvent))
                )
            else:
                if callable(raw_callback):
                    POOL.submit(
                        raw_callback(up)
                    )

    def __get_out_peer_and_message(self, peer: peers_pb2.Peer, text: str,
                                   interactive_media_groups: List[InteractiveMediaGroup]) \
            -> Tuple[peers_pb2.OutPeer, messaging_pb2.MessageContent]:
        out_peer = self.manager.get_out_peer(peer)
        msg = messaging_pb2.MessageContent()
        msg.textMessage.text = text
        if interactive_media_groups is not None:
            for g in interactive_media_groups:
                media = msg.textMessage.media.add()
                g.render(media)
        return out_peer, msg

    @staticmethod
    def __get_medias(medias: List[MessageMedia]) -> List[messaging_pb2.MessageMedia] or None:
        for i in range(len(medias)):
            if medias[i].audio and isinstance(medias[i].audio.audio.file_location, AsyncTask):
                medias[i].audio.audio.file_location = medias[i].audio.audio.file_location.wait()
            if medias[i].image and isinstance(medias[i].image.image.file_location, AsyncTask):
                medias[i].image.image.file_location = medias[i].image.image.file_location.wait()
            if medias[i].web_page and medias[i].web_page.image and \
                    isinstance(medias[i].web_page.image.file_location, AsyncTask):
                medias[i].web_page.image.file_location = medias[i].web_page.image.file_location.wait()
        return [x.to_api() for x in medias]

    def __send_message(self, request: messaging_pb2.RequestSendMessage) -> UUID:
        return UUID.from_api(self.internal.messaging.SendMessage(request).message_id)

    def __update(self, message: Message, new_message: messaging_pb2.MessageContent) -> None:
        if hasattr(message, "mid"):
            mid = message.mid
        else:
            raise AttributeError("message has not attribute message_id or mid")

        if message.edited_at:
            last_edited_at = message.edited_at
        else:
            last_edited_at = message.date

        request = messaging_pb2.RequestUpdateMessage(
            mid=mid.to_api(),
            updated_message=new_message,
            last_edited_at=last_edited_at
        )
        self.internal.messaging.UpdateMessage(request)



from dialog_api import messaging_pb2
from google.protobuf.wrappers_pb2 import StringValue

from dialog_bot_sdk.entities.media.ImageMedia import ImageLocation


class WebPageMedia:
    def __init__(self, url: str, title: str = "", description: str = "", image: ImageLocation = None):
        self.url = url
        self.title = title
        self.description = description
        self.image = image

    def to_api(self) -> messaging_pb2.WebpageMedia:
        return messaging_pb2.WebpageMedia(url=StringValue(value=self.url), title=StringValue(value=self.title),
                                          description=StringValue(value=self.description), image=self.image.to_api())

    @classmethod
    def from_api(cls, web_page: messaging_pb2.WebpageMedia) -> 'WebPageMedia':
        return cls(web_page.url, web_page.title, web_page.description, ImageLocation.from_api(web_page.image))

    def __dict__(self):
        return {"url": self.url, "title": self.title, "description": self.description, "image": self.image.__dict__()}

    def __str__(self):
        return "WebPageMedia({})".format(self.__dict__())



import unittest

from dialog_api import peers_pb2

from dialog_bot_sdk.entities.FullUser import FullUser
from dialog_bot_sdk.entities.User import User
from dialog_bot_sdk.utils import AsyncTask
from tests.bot import bot
from tests.test_classes.messaging import Messaging
from tests.test_classes.search import Search
from tests.test_classes.updates import Updates
from tests.test_classes.users import Users


class TestUsers(unittest.TestCase):
    out_peer = peers_pb2.OutPeer(
        type=peers_pb2.PEERTYPE_PRIVATE,
        id=0,
        access_hash=0
    )
    bot.internal.search = Search()
    bot.internal.updates = Updates()
    bot.internal.messaging = Messaging()
    bot.internal.users = Users()

    def test_get_user_by_nick(self):
        user = bot.users.get_user_by_nick("name")
        self.assertTrue(isinstance(user, AsyncTask))
        self.assertIsNone(user.wait())
        user = bot.users.get_user_by_nick("nick")
        self.assertTrue(isinstance(user, AsyncTask))
        self.assertTrue(isinstance(user.wait(), User))

    def test_get_user_by_id(self):
        user = bot.users.get_user_by_id(0)
        self.assertTrue(isinstance(user, AsyncTask))
        self.assertIsNone(user.wait())
        user = bot.users.get_user_by_id(1)
        self.assertTrue(isinstance(user, AsyncTask))
        self.assertTrue(isinstance(user.wait(), User))

    def test_search_users_by_nick_substring(self):
        user = bot.users.search_users_by_nick_substring("name")
        self.assertTrue(isinstance(user, AsyncTask))
        self.assertEqual(user.wait(), [])
        user = bot.users.search_users_by_nick_substring("nick")
        self.assertTrue(isinstance(user, AsyncTask))
        self.assertTrue(isinstance(user.wait()[0], User))

    def test_get_full_profile_by_nick(self):
        user = bot.users.get_full_profile_by_nick("nick")
        self.assertTrue(isinstance(user, AsyncTask))
        self.assertTrue(isinstance(user.wait(), FullUser))

    def test_get_full_profile_by_id(self):
        user = bot.users.get_full_profile_by_id(0)
        self.assertTrue(isinstance(user, AsyncTask))
        self.assertIsNone(user.wait())


if __name__ == '__main__':
    unittest.main()



import unittest

from dialog_api.sequence_and_updates_pb2 import ResponseGetDifference

from dialog_bot_sdk.utils import AsyncTask
from tests.bot import bot
from tests.test_classes.updates import Updates


class TestUpdates(unittest.TestCase):
    bot.internal.updates = Updates()

    def test_get_difference(self):
        seq = bot.updates.get_difference(0)
        self.assertTrue(isinstance(seq, AsyncTask))
        self.assertTrue(isinstance(seq.wait(), ResponseGetDifference))

    def test_get_state(self):
        seq = bot.updates.get_state()
        self.assertTrue(isinstance(seq, AsyncTask))
        self.assertEqual(seq.wait(), 0)


if __name__ == '__main__':
    unittest.main()



import unittest

from dialog_api import peers_pb2
from mock import patch

from dialog_bot_sdk.entities.Avatar import Avatar
from dialog_bot_sdk.entities.Group import Group
from dialog_bot_sdk.entities.Peer import Peer, PeerType
from dialog_bot_sdk.entities.User import User
from dialog_bot_sdk.utils import AsyncTask
from tests.bot import bot
from tests.test_classes.groups import Groups
from tests.test_classes.media_and_files import MediaAndFiles, Put
from tests.test_classes.search import Search
from tests.test_classes.updates import Updates


class TestGroups(unittest.TestCase):
    group_peer = Peer(id=0, type=PeerType.PEERTYPE_GROUP)
    user1 = Peer(id=0, type=PeerType.PEERTYPE_PRIVATE)
    test_file = "../dialog_bot_sdk/examples/files/example.png"

    bot.internal.groups = Groups()
    bot.internal.search = Search()
    bot.internal.media_and_files = MediaAndFiles()
    bot.internal.updates = Updates()

    def test_create_group(self):
        group = bot.groups.create_public_group("title", "short_name")
        self.assertTrue(isinstance(group, AsyncTask))
        self.assertTrue(isinstance(group.wait(), Group))
        group = bot.groups.create_private_group("title")
        self.assertTrue(isinstance(group, AsyncTask))
        self.assertTrue(isinstance(group.wait(), Group))
        group = bot.groups.create_public_channel("title", "short_name")
        self.assertTrue(isinstance(group, AsyncTask))
        self.assertTrue(isinstance(group.wait(), Group))
        group = bot.groups.create_private_channel("title")
        self.assertTrue(isinstance(group, AsyncTask))
        self.assertTrue(isinstance(group.wait(), Group))

    def test_find_group_by_short_name(self):
        group = bot.groups.find_group_by_short_name("title")
        self.assertTrue(isinstance(group, AsyncTask))
        self.assertIsNone(group.wait())
        bot.manager.add_out_peer(peers_pb2.OutPeer(id=1, access_hash=1, type=PeerType.PEERTYPE_GROUP))
        group = bot.groups.find_group_by_short_name("short_name")
        self.assertTrue(isinstance(group, AsyncTask))
        self.assertTrue(isinstance(group.wait(), Group))

    def test_find_group_by_id(self):
        group = bot.groups.find_group_by_id(1)
        self.assertTrue(isinstance(group, AsyncTask))
        self.assertIsNone(group.wait())
        bot.manager.add_out_peer(peers_pb2.OutPeer(id=1, access_hash=1, type=PeerType.PEERTYPE_GROUP))
        group = bot.groups.find_group_by_id(1)
        self.assertTrue(isinstance(group, AsyncTask))
        self.assertTrue(isinstance(group.wait(), Group))

    def test_load_members(self):
        members = bot.groups.load_members(self.group_peer, 2)
        self.assertTrue(isinstance(members, AsyncTask))
        self.assertTrue(isinstance(members.wait()[0], User))

    def test_kick_user(self):
        kick = bot.groups.kick_user(self.group_peer, self.user1)
        self.assertTrue(isinstance(kick, AsyncTask))
        self.assertIsNone(kick.wait())

    def test_invite_user(self):
        invite = bot.groups.invite_user(self.group_peer, self.user1)
        self.assertTrue(isinstance(invite, AsyncTask))
        self.assertIsNone(invite.wait())

    def test_set_default_group_permissions(self):
        permissions = bot.groups.set_default_group_permissions(self.group_peer)
        self.assertTrue(isinstance(permissions, AsyncTask))
        self.assertIsNone(permissions.wait())

    def test_set_member_permissions(self):
        permissions = bot.groups.set_member_permissions(self.group_peer, self.user1)
        self.assertTrue(isinstance(permissions, AsyncTask))
        self.assertIsNone(permissions.wait())

    def test_get_group_member_permissions(self):
        permissions = bot.groups.get_group_member_permissions(self.group_peer, [self.user1])
        self.assertTrue(isinstance(permissions, AsyncTask))
        self.assertEqual(permissions.wait(), [])

    def test_edit_group_title(self):
        title = bot.groups.edit_group_title(self.group_peer, "title")
        self.assertTrue(isinstance(title, AsyncTask))
        self.assertIsNone(title.wait())

    @patch('requests.put')
    def test_edit_avatar(self, put):
        put.return_value = Put(200)
        avatar = bot.groups.edit_avatar(self.group_peer, self.test_file)
        self.assertTrue(isinstance(avatar, AsyncTask))
        self.assertTrue(isinstance(avatar.wait(), Avatar))
        put.return_value = Put(400)
        avatar = bot.groups.edit_avatar(self.group_peer, self.test_file)
        self.assertTrue(isinstance(avatar, AsyncTask))
        self.assertIsNone(avatar.wait())

    def test_remove_group_avatar(self):
        remove = bot.groups.remove_group_avatar(self.group_peer)
        self.assertTrue(isinstance(remove, AsyncTask))
        self.assertIsNone(remove.wait())

    def test_edit_group_about(self):
        about = bot.groups.edit_group_about(self.group_peer, "about")
        self.assertTrue(isinstance(about, AsyncTask))
        self.assertIsNone(about.wait())

    def test_leave_group(self):
        leave = bot.groups.leave_group(self.group_peer)
        self.assertTrue(isinstance(leave, AsyncTask))
        self.assertIsNone(leave.wait())

    def test_make_user_admin(self):
        admin = bot.groups.make_user_admin(self.group_peer, self.user1, None)
        self.assertTrue(isinstance(admin, AsyncTask))
        self.assertIsNone(admin.wait())

    def test_transfer_ownership(self):
        owner = bot.groups.transfer_ownership(self.group_peer, self.user1)
        self.assertTrue(isinstance(owner, AsyncTask))
        self.assertIsNone(owner.wait())

    def test_get_group_invite_url(self):
        url = bot.groups.get_group_invite_url(self.group_peer)
        self.assertTrue(isinstance(url, AsyncTask))
        self.assertEqual(url.wait(), "url")

    def test_get_group_invite_url_base(self):
        url = bot.groups.get_group_invite_url_base()
        self.assertTrue(isinstance(url, AsyncTask))
        self.assertEqual(url.wait(), "url")

    def test_revoke_invite_url(self):
        url = bot.groups.revoke_invite_url(self.group_peer)
        self.assertTrue(isinstance(url, AsyncTask))
        self.assertEqual(url.wait(), "url")

    def test_join_group(self):
        group = bot.groups.join_group("url")
        self.assertTrue(isinstance(group, AsyncTask))
        self.assertTrue(isinstance(group.wait(), Group))

    def test_join_group_by_peer(self):
        group = bot.groups.join_group_by_peer(self.group_peer)
        self.assertTrue(isinstance(group, AsyncTask))
        self.assertIsNone(group.wait())


if __name__ == '__main__':
    unittest.main()
